import { jsx as _jsx } from "react/jsx-runtime";
/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { useState, useEffect } from 'react';
import { CodeCell, MarkdownCell } from '@jupyterlab/cells';
import { Box } from '@primer/react';
import CellAdapter from './CellAdapter';
import Lumino from '../lumino/Lumino';
import { useJupyter } from './../../jupyter/JupyterContext';
import useCellStore from './CellState';
export const Cell = (props) => {
    const { type = 'code', source = '', autoStart, showToolbar = true } = props;
    const { serverSettings, defaultKernel } = useJupyter();
    const cellStore = useCellStore();
    const [adapter, setAdapter] = useState();
    const handleCellInitEvents = (adapter) => {
        adapter.cell.model.contentChanged.connect((cellModel, changedArgs) => {
            cellStore.setSource(cellModel.sharedModel.getSource());
        });
        if (adapter.cell instanceof CodeCell) {
            adapter.cell.outputArea.outputLengthChanged?.connect((outputArea, outputsCount) => {
                cellStore.setOutputsCount(outputsCount);
            });
        }
        adapter.sessionContext.initialize().then(() => {
            if (!autoStart) {
                return;
            }
            // Perform auto-start for code or markdown cells
            if (adapter.cell instanceof CodeCell) {
                const execute = CodeCell.execute(adapter.cell, adapter.sessionContext);
                execute.then((msg) => {
                    cellStore.setKernelAvailable(true);
                });
            }
            if (adapter.cell instanceof MarkdownCell) {
                adapter.cell.rendered = true;
            }
        });
    };
    useEffect(() => {
        if (defaultKernel && serverSettings) {
            defaultKernel.ready.then(() => {
                const adapter = new CellAdapter({
                    type,
                    source,
                    serverSettings,
                    kernel: defaultKernel,
                    boxOptions: { showToolbar }
                });
                cellStore.setAdapter(adapter);
                cellStore.setSource(source);
                handleCellInitEvents(adapter);
                setAdapter(adapter);
                const handleDblClick = (event) => {
                    let target = event.target;
                    /**
                     * Find the DOM searching by the markdown output class (since child elements can be clicked also)
                     * If a rendered markdown was found, then back cell to editor mode
                     */
                    while (target && !target.classList.contains('jp-MarkdownOutput')) {
                        target = target.parentElement;
                    }
                    if (target && target.classList.contains('jp-MarkdownOutput')) {
                        adapter.cell.rendered = false;
                    }
                };
                // Adds the event for double click and the removal on component's destroy
                document.addEventListener('dblclick', handleDblClick);
                return () => {
                    document.removeEventListener('dblclick', handleDblClick);
                };
            });
        }
    }, [source, defaultKernel, serverSettings]);
    return adapter ? (_jsx(Box, { sx: {
            '& .dla-Jupyter-Cell': {
                position: 'relative !important',
                contain: 'content !important',
            },
            '& .jp-Toolbar': {
                height: 'auto !important',
                position: 'relative',
            },
            '& .lm-BoxPanel': {
                position: 'relative',
            },
            '& .jp-Cell': {
                position: 'relative',
            },
            '& .jp-CodeCell': {
                height: 'auto !important',
                position: 'relative',
            },
            '& .jp-MarkdownCell': {
                height: 'auto !important',
                minHeight: '65px',
                position: 'relative',
            },
            '& .jp-Cell-outputArea': {
                paddingBottom: '30px',
            },
            '& .jp-CodeMirrorEditor': {
                cursor: 'text !important',
            },
        }, children: _jsx(Lumino, { children: adapter.panel }) })) : (_jsx(Box, { children: "Loading Jupyter Cell..." }));
};
Cell.defaultProps = {
    source: '',
    autoStart: true,
};
export default Cell;
//# sourceMappingURL=Cell.js.map